package be.iminds.ilabt.jfed.util;

import java.time.Clock;
import java.time.Instant;
import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * TimeUtil: time related helpers
 */
public class TimeUtil {
    private TimeUtil() {}

    public static String calcAgo(Date date) {
        return calcAgo(date.toInstant());
    }

    public static String calcAgo(Instant date) {
        return calcAgo(date, Instant.now());
    }

    /** return a human representation of a time in the future.
     * Example: "in 5 minutes" */
    public static String calcIn(Instant date) {
        return calcIn(date, Instant.now());
    }

    /** return a human representation of a time in the future.
     * Example: "in 5 minutes" */
    public static String calcIn(Instant date, Instant now) {
        long ms = date.toEpochMilli() - now.toEpochMilli();

        if (ms < -5*1000l) return "in the past";

        if (ms < 4*1000l) return "almost now";
        if (ms < 20*1000l) return "in a few seconds";
        if (ms < 60*1000l) return "in less than a minute";

        long minutes = TimeUnit.MILLISECONDS.toMinutes(ms);
        if (minutes == 1) return "in one minute";
        if (minutes < 60) return "in " + minutes+" minutes";

        long hours = TimeUnit.MILLISECONDS.toHours(ms);
        if (hours == 1) return "in one hour";
        if (hours < 28) return "in " + hours+" hours";

        long days = TimeUnit.MILLISECONDS.toDays(ms);
        if (days < 61) return "in "+days+" days";
        if (days < (365*2)) return "in more than "+Math.floor(days/30.0)+" months";
        return "in more than "+Math.floor(days/356.0)+" years";
    }
    public static String calcAgo(Instant date, Instant now) {
        long ms = now.toEpochMilli() - date.toEpochMilli();

        if (ms < -5*1000l) return /*"in the future?!"*/calcIn(date, now);

        if (ms < 4*1000l) return "just now";
        if (ms < 20*1000l) return "a few seconds ago";
        if (ms < 60*1000l) return "less than a minute ago";

        long minutes = TimeUnit.MILLISECONDS.toMinutes(ms);
        if (minutes < 60) return minutes+" minutes ago";

        long hours = TimeUnit.MILLISECONDS.toHours(ms);
        if (hours < 28) return hours+" hours ago";

        long days = TimeUnit.MILLISECONDS.toDays(ms);
        if (days < 61) return days+" days ago";
        if (days < (365*2)) return "more than "+Math.floor(days/30.0)+" months ago";
        return "more than "+Math.floor(days/356.0)+" years ago";
    }
}
